SystemOrganization addCategory: #'Seaside-Email'!

WAObject subclass: #WAEmailAddress
	instanceVariableNames: 'username address'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Email'!

!WAEmailAddress class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 13:01'!
address: aString
	^self new
		address: aString;
		yourself! !

!WAEmailAddress class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 13:01'!
address: anAddressString username: aUsernameString
	^self new
		address: anAddressString;
		username: aUsernameString;
		yourself! !

!WAEmailAddress methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:01'!
address
	^address! !

!WAEmailAddress methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:36'!
address: aString
	address := self sanitize: aString! !

!WAEmailAddress methodsFor: 'displaying' stamp: 'pmm 9/2/2008 14:14'!
encodeOn: aDocument
	self encodeOn: aDocument usingQuotes: false! !

!WAEmailAddress methodsFor: 'private' stamp: 'pmm 9/2/2008 14:14'!
encodeOn: aStream usingQuotes: aBoolean
	username isNil ifFalse:  [
		aBoolean ifTrue: [ aStream nextPut: $" ].
		aStream nextPutAll: username.
		aBoolean ifTrue: [ aStream nextPut: $" ].
		aStream nextPutAll: ' <' ].
	aStream nextPutAll: address.
	username isNil ifFalse: [ aStream nextPut: $> ]! !

!WAEmailAddress methodsFor: 'printing' stamp: 'pmm 8/31/2008 13:01'!
printOn: aStream 
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self seasideString.
	aStream nextPut: $)! !

!WAEmailAddress methodsFor: 'private' stamp: 'pmm 8/31/2008 13:37'!
sanitize: aString
	"prevents SMTP header injection"
	| newlineIndex |
	newlineIndex := (aString indexOf: Character cr) min: (aString indexOf: Character lf).
	^newlineIndex = 0
		ifTrue: [ aString ]
		ifFalse: [ aString copyFrom: 1 to: newlineIndex - 1  ]! !

!WAEmailAddress methodsFor: 'displaying' stamp: 'pmm 9/2/2008 14:15'!
seasideString
	^ String streamContents: [ :stream | 
		self encodeOn: stream usingQuotes: true ]! !

!WAEmailAddress methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:01'!
username
	^username! !

!WAEmailAddress methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:37'!
username: aString
	username := self sanitize: aString! !

WAObject subclass: #WAEmailBody
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Email'!

WAEmailBody subclass: #WAStringEmailBody
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Email'!

!WAStringEmailBody class methodsFor: 'private' stamp: 'pmm 8/31/2008 15:43'!
convertCrLfToCrIn: aString
	^aString
		copyReplaceAll: (String with: Character cr with: Character lf)
		with: (String with: Character cr)! !

!WAStringEmailBody class methodsFor: 'private' stamp: 'pmm 8/31/2008 18:56'!
splitIntoLines: aString
	^Array streamContents: [ :stream |
		| cr start |
		cr := Character cr.
		start := 1.
		1 to: aString size do: [ :index |
			(aString at: index) = cr ifTrue: [
				stream nextPut: (aString copyFrom: start to: index - 1).
				start := index + 1 ] ].
			start <= (aString size + 1) ifTrue: [ 
				stream nextPut: (aString copyFrom: start to: aString size) ] ]! !

!WAStringEmailBody class methodsFor: 'private' stamp: 'pmm 8/31/2008 16:12'!
splitString: aString intoChunksNoLongerThan: anInteger
	^Array streamContents: [ :stream |
		self
			splitString: aString
			intoChunksNoLongerThan: anInteger
			into: stream ]! !

!WAStringEmailBody class methodsFor: 'private' stamp: 'pmm 8/31/2008 18:34'!
splitString: aString intoChunksNoLongerThan: anInteger into: aStream
	| space lastSpaceIndex toScan |
	space := Character space.
	toScan := aString size min: anInteger + 1.
	lastSpaceIndex := 0.
	1 to: toScan do: [ :index |
		(aString at: index) = space ifTrue: [
			lastSpaceIndex := index ] ].
	
	lastSpaceIndex = 0
		ifTrue: [
			lastSpaceIndex := aString size min: anInteger.
			aStream nextPut: (aString copyFrom: 1 to: lastSpaceIndex).
			lastSpaceIndex < aString size ifTrue: [
				self
					splitString: (aString copyFrom: lastSpaceIndex + 1 to: aString size)
					intoChunksNoLongerThan: anInteger
					into: aStream ] ]
		ifFalse: [
			aStream nextPut: (aString copyFrom: 1 to: lastSpaceIndex - 1).
			lastSpaceIndex < aString size ifTrue: [
				self
					splitString: (aString copyFrom: lastSpaceIndex + 1 to: aString size)
					intoChunksNoLongerThan: anInteger
					into: aStream ] ]! !

!WAStringEmailBody class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 15:43'!
string: aString
	^self new
		string: (self convertCrLfToCrIn: aString);
		yourself! !

!WAStringEmailBody class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 18:48'!
string: aString lineLength: anInteger
	| string |
	string := String streamContents: [ :stream |
		| lines |
		lines := self splitIntoLines: (self convertCrLfToCrIn: aString).
		lines do: [ :each |
			each size <= anInteger
				ifTrue: [ stream nextPutAll: each ]
				ifFalse: [
					(self splitString: each intoChunksNoLongerThan: anInteger)
						do: [ :chunk | stream nextPutAll: chunk ]
						separatedBy: [ stream cr ] ] ]
			separatedBy: [ stream cr ] ].
	^self new
		string: string;
		yourself! !

!WAStringEmailBody methodsFor: 'accessing' stamp: 'pmm 8/31/2008 15:36'!
seasideString
	^self string! !

!WAStringEmailBody methodsFor: 'accessing' stamp: 'pmm 8/31/2008 15:29'!
string
	^ string! !

!WAStringEmailBody methodsFor: 'accessing' stamp: 'pmm 8/31/2008 15:29'!
string: aString
	string := aString! !

WAObject subclass: #WAEmailMessage
	instanceVariableNames: 'to from cc bcc subject headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Email'!

!WAEmailMessage class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 15:33'!
from: aFromEmailAddress to: aToEmailAddress subject: aString
	^self new
		from: aFromEmailAddress;
		addTo: aToEmailAddress;
		subject: aString;
		yourself! !

!WAEmailMessage class methodsFor: 'instance creation' stamp: 'pmm 8/31/2008 15:33'!
from: aFromEmailAddress toAll: aCollectionOfEmailAddress subject: aString
	^self new
		from: aFromEmailAddress;
		addAllTo: aCollectionOfEmailAddress;
		subject: aString;
		yourself! !

!WAEmailMessage methodsFor: 'adding' stamp: 'pmm 8/31/2008 13:44'!
addAllTo: aCollectionOfEmailAddress
	aCollectionOfEmailAddress do: [ :each |
		self addTo: each ]! !

!WAEmailMessage methodsFor: 'adding' stamp: 'pmm 8/31/2008 13:41'!
addBcc: anEmailAddress
	^self bcc add: anEmailAddress! !

!WAEmailMessage methodsFor: 'adding' stamp: 'pmm 8/31/2008 13:40'!
addCc: anEmailAddress
	^self cc add: anEmailAddress! !

!WAEmailMessage methodsFor: 'adding' stamp: 'pmm 8/31/2008 13:25'!
addTo: anEmailAddress
	^self to add: anEmailAddress! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
bcc
	^ bcc! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
body
	^ body! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
body: anObject
	body := anObject! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
cc
	^ cc! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
from
	^ from! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
from: anObject
	from := anObject! !

!WAEmailMessage methodsFor: 'adding' stamp: 'pmm 8/31/2008 20:45'!
headerAt: aKeyString put: aValueString
	^self headers at: aKeyString put: aValueString! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
headers
	^ headers! !

!WAEmailMessage methodsFor: 'initialize-release' stamp: 'pmm 8/31/2008 20:47'!
initialize
	super initialize.
	to := OrderedCollection new.
	cc := OrderedCollection new.
	bcc := OrderedCollection new.
	headers := Dictionary new.
	self headerAt: 'Date' put: (WAPrinter rfc822 print: DateAndTime now)! !

!WAEmailMessage methodsFor: 'private' stamp: 'pmm 8/31/2008 15:37'!
plainMessage
	^String streamContents: [ :stream |
		
		self
			renderAddress: self from
			withHeander: 'From: '
			on: stream.
			
		self to do: [ :each |
			self
				renderAddress: each
				withHeander: 'To: '
				on: stream ].
			
		self cc do: [ :each |
			self
				renderAddress: each
				withHeander: 'Cc: '
				on: stream ].
			
		self bcc do: [ :each |
			self
				renderAddress: each
				withHeander: 'Bcc: '
				on: stream ].
		stream
			nextPutAll: 'Subject: ';
			nextPutAll: self subject;
			cr; cr.
			
		self body isNil ifFalse: [
			stream nextPutAll: self body seasideString ] ]! !

!WAEmailMessage methodsFor: 'private' stamp: 'pmm 8/31/2008 20:16'!
receipents
	^(self to, self cc, self bcc) collect: [ :each |
		each seasideString ]! !

!WAEmailMessage methodsFor: 'private' stamp: 'pmm 8/31/2008 15:13'!
renderAddress: anEmailAddress withHeander: aString on: aStream
	aStream
		nextPutAll: aString;
		nextPutAll: anEmailAddress seasideString;
		cr! !

!WAEmailMessage methodsFor: 'actions' stamp: 'pmm 8/31/2008 20:16'!
send
	SeasidePlatformSupport
		deliverMailFrom: self from seasideString
		to: self receipents
		text: self plainMessage! !

!WAEmailMessage methodsFor: 'convenience' stamp: 'pmm 8/31/2008 15:32'!
setStringBody: aString wrap: aBoolean
	self body: (aBoolean
		ifTrue: [ WAStringEmailBody string: aString lineLength: 80 ]
		ifFalse: [ WAStringEmailBody string: aString ])! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
subject
	^ subject! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
subject: anObject
	subject := anObject! !

!WAEmailMessage methodsFor: 'accessing' stamp: 'pmm 8/31/2008 13:12'!
to
	^ to! !
